"use strict";
this.name        = "Naval Grid";
this.author	 = "Lone_Wolf";
this.copyright	 = "2015 Lone_Wolf";
this.licence     = "CC-by-SA 4.0";
this.description = "Enhanced shield recharging unit";
this.version	 = "1.10";

this._ng_charging_devices = 
  [
    "EQ_SHIELD_BOOSTER",
    "EQ_NAVAL_SHIELD_BOOSTER",
    "EQ_ENERGY_UNIT",
    "EQ_NAVAL_ENERGY_UNIT"
  ];

this._ng_shield_charge_modifiers = 
  {
    fwd:0,
    aft:0
  };

this._ng_switch_max = 0;

this.startUpComplete = function()
  {
    if ( !player.ship.script ) { player.ship.setScript("oolite-default-ship-script.js"); }; // just in case
    if ( player.ship.equipmentStatus("EQ_ENERGY_GRID") == "EQUIPMENT_OK") 
      { 
        this._ng_switch_max = 0.5;
        this._ng_setup_handlers();
      };
    if ( player.ship.equipmentStatus("EQ_NAVAL_GRID") == "EQUIPMENT_OK")
      { 
        this._ng_switch_max = 0.9;
        this._ng_setup_handlers();
      };
    delete this.startupComplete;
  };

  
this._ng_setup_handlers = function()
  {
    // store existing handlers
    if ( player.ship.script.equipmentAdded ) { player.ship.script._ng_player_equipmentAdded_original = player.ship.script.equipmentAdded; };
    if ( player.ship.script.equipmentRemoved ) { player.ship.script._ng_player_equipmentRemoved_original = player.ship.script.equipmentRemoved; };
    // set new handlers
    var new_player_equipmentAdded = function(eqkey)
      {
        worldScripts["Naval Grid"]._ng_equipment(eqkey, player.ship.script._ng_player_equipmentAdded_original ); 
      };
    var new_player_equipmentRemoved = function(eqkey)
      {
        worldScripts["Naval Grid"]._ng_equipment(eqkey, player.ship.script._ng_player_equipmentRemoved_original ); 
      };
    player.ship.script.equipmentAdded = new_player_equipmentAdded;
    player.ship.script.equipmentRemoved = new_player_equipmentRemoved;
  };

this._ng_player_equipment = function (eqkey, original_handler)
  {
    // avoiding "this.xxx" cause function will be called as a method of player.ship.script
    // only act if STATUS_IN_FLIGHT
    if ( player.ship.status != STATUS_IN_FLIGHT ) { original_handler(eqkey); return };
    // verify if added/removed device influences NG modifiers
    if ( worldScripts["Naval Grid"]._ng_charging_devices.indexOf(eqkey) == -1 ) { original_handler(eqkey); return };
    // revert ng modifiers
    worldScripts["Naval Grid"]._ng_player_modifiers("revert");
    // apply original handler for device
    original_handler(eqkey);
    worldScripts["Naval Grid"]._ng_player_modifiers( "apply");
  };

this._ng_player_modifiers = function(type)
  {
    // type = string
    //   apply = add values
    //   revert = subtract values
    // verify if energy grid or naval grid are present and undamaged
    if ( this._ng_switch_max == 0 ) return;
    var rates =
      { 
        fwd: player.ship.forwardShieldRechargeRate,
        aft: player.ship.aftShieldRechargeRate,
        energy: player.ship.energyRechargeRate,
        switch_max: this._ng_switch_max
      };
    var modifiers = { fwd: 0,aft: 0 };
    modifiers = this._ng_calc_modifiers(rates);
    switch ( type )
      {
        case "apply":
          player.ship.forwardShieldRechargeRate += modifiers.fwd;
          player.ship.aftShieldRechargeRate += modifiers.fwd;
          break;
        case "revert":
          player.ship.forwardShieldRechargeRate -= modifiers.fwd;
          player.ship.aftShieldRechargeRate -= modifiers.fwd;
          break;
        default:
          break;
      };
  };
  
this._ng_calc_modifiers = function(charge_rates)
  {
    /* Input
      object charge_rates = { fwd_charge, aft_charge, energy_charge, switch_max }
    Output
      object = { fwd_modifier, aft_modifier }
    */
    var modifiers = { fwd: 0, aft: 0 };
    var shield_charge = fwd_charge + aft_charge;
    charge_room = switch_max * ( energy_charge - shield_charge );
    modifiers.fwd = (fwd_charge / shield_charge) * charge_room;
    modifiers.aft = (aft_charge / shield_charge) * charge_room;
    return modifiers;
  };
